from proxmoxer import ProxmoxAPI
import time
import re
import sys
import paramiko
import subprocess
from enums import Timeout

TEMPLATES = {
    "1": "local:vztmpl/centos-8-sshd.tar.gz",
    "2": "local:vztmpl/ubuntu-22-sshd.tar.gz"
}

def test_proxmox_connection():
    try:
        nodes = proxmox.nodes.get()
        print("Connection successful. Nodes fetched:")
        for node in nodes:
            print(f"- {node['node']}")
    except Exception as e:
        print(f"Failed to connect: {e}")
        exit(1)

def get_next_vmid():
    vmids = [
        int(ct["vmid"])
        for ct in proxmox.nodes("Debian-1201-bookworm-amd64-base").lxc.get()
    ]
    return max(vmids) + 1 if vmids else 100

def get_all_containers_with_ips(node):
    containers = proxmox.nodes(node).lxc.get()
    container_info = []
    for ct in containers:
        vmid = ct.get("vmid")
        name = ct.get("name", f"VMID-{vmid}")
        config = proxmox.nodes(node).lxc(vmid).config.get()
        net0 = config.get("net0", "")
        ip_match = re.search(r"ip=([\d\.]+)", net0)
        ip_address = ip_match.group(1) if ip_match else "No IP Assigned"
        container_info.append((vmid, name, ip_address))
    return container_info

def create_container(node, vmid, template, password, ip_address, resources, storage="local"):
    hostname = f"pct{vmid}"
    proxmox.nodes(node).lxc.create(
        vmid=vmid,
        ostemplate=template,
        hostname=hostname,
        password=password,
        storage=storage,
        memory=resources['memory'],
        swap=resources['swap'],
        cores=resources['cores'],
        net0=f"name=ens18,bridge=vmbr1,ip={ip_address}/24,gw=172.16.0.1",
        rootfs=f"{storage}:{resources['rootfs']}",
    )
    print(f"Container {vmid} ({hostname}) created successfully.")
    return hostname

def start_container(node, vmid):
    print(f"Starting container {vmid}...")
    for attempt in range(1, 6):
        try:
            proxmox.nodes(node).lxc(vmid).status.start.post()
            print(f"Container {vmid} started successfully.")
            return
        except Exception as e:
            print(f"Attempt {attempt}: Failed to start container {vmid}: {e}")
            if attempt < 5:
                print("Retrying in 5 seconds...")
                time.sleep(5)
            else:
                print(f"Exceeded maximum retries to start container {vmid}.")
                return

def create_superuser(ip_address, username, userpass, rootpassword):
    print(f"Creating superuser {username} on container with IP {ip_address}...")

    # Remove IP from known_hosts to avoid SSH errors
    try:
        subprocess.run(["ssh-keygen", "-R", ip_address], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except subprocess.CalledProcessError as e:
        print(f"Failed to clear IP from known_hosts: {e}")

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    try:
        ssh.connect(ip_address, username='root', password=rootpassword)
        stdin, stdout, stderr = ssh.exec_command(f"useradd -m -s /bin/bash {username} && echo '{username}:{userpass}' | chpasswd && usermod -aG sudo {username}")
        stdout.channel.recv_exit_status()
        print(f"Superuser {username} created successfully.")
    except Exception as e:
        print(f"Failed to create superuser: {e}")
    finally:
        ssh.close()

def main():
    if len(sys.argv) < 12:
        print("Usage: python3 init.py <proxmox_hostname> <proxmox_username> <proxmox_token_name> <proxmox_token_value> <verify_ssl> <template_choice> <rootpassword> <memory> <cores> <storage> <IP> [<newuser> <newuserpass>]")
        exit(1)

    template_choice = sys.argv[6]
    rootpassword = sys.argv[7]

    if template_choice not in TEMPLATES:
        print(f"Invalid template choice {template_choice}. Use 1 for CentOS 8 or 2 for Ubuntu 22.")
        exit(1)

    try:
        memory_gb = int(sys.argv[8])
        cores = int(sys.argv[9])
        storage_gb = int(sys.argv[10])
        ip_address = sys.argv[11]

        # Validate limits
        if not (1 <= memory_gb <= 16):
            raise ValueError(f"RAM must be between 1 and 16 GB.Your input are: {memory_gb} GB")
        if not (1 <= cores <= 16):
            raise ValueError(f"CPU Cores must be between 1 and 16.Your input are: {cores} cores")
        if not (10 <= storage_gb <= 1000):
            raise ValueError(f"Disk space must be between 10 and 1000 GB.Your input are: {storage_gb} GB")

    except ValueError as e:
        print(f"Invalid parameter: {e}")
        exit(1)

    # Optional parameters
    
    if len(sys.argv) > 12:
        username = sys.argv[12]
    else:
        username = None
    
    if len(sys.argv) > 13:
        userpass = sys.argv[13]
    else:
        userpass = None

    memory = memory_gb * 1024  # Convert GB to MB
    swap = memory_gb * 512     # Swap is half of memory
    storage = f"local:{storage_gb}"  # Define storage size

    host = sys.argv[1]
    user = sys.argv[2]
    token_name = sys.argv[3]
    token_value = sys.argv[4]
    verify_ssl = True if 'true' in sys.argv[5] else False
    print(host, user, token_name, token_value, verify_ssl)

    global proxmox
    proxmox = ProxmoxAPI(
        host=host,
        user=user,
        token_name=token_name,
        token_value=token_value,
        verify_ssl=verify_ssl,
        timeout=Timeout.SERVICE_SSH_TCP.value
    )

    test_proxmox_connection()

    node = "Debian-1201-bookworm-amd64-base"
    storage_pool = "local"

    # Fetch existing containers and their IPs
    containers = get_all_containers_with_ips(node)
    existing_ips = [ip for _, _, ip in containers if ip != "No IP Assigned"]

    if ip_address in existing_ips:
        print(f"IP address {ip_address} is already in use. Please choose another.")
        exit(1)

    template = TEMPLATES[template_choice]
    vmid = get_next_vmid()

    print(f"Creating container with VMID {vmid}...")
    hostname = create_container(
        node=node,
        vmid=vmid,
        template=template,
        password=rootpassword,
        ip_address=ip_address,
        resources={'memory': memory, 'swap': swap, 'cores': cores, 'rootfs': storage_gb},
        storage=storage_pool
    )
    print(f"Container hostname: {hostname}")

    start_container(node, vmid)

    if username and userpass:
        create_superuser(ip_address, username, userpass, rootpassword)

if __name__ == "__main__":
    main()